from typing import NamedTuple


class Author(NamedTuple):
    email: str = None
    name: str = None


class Category(NamedTuple):
    term: str = None
    label: str = None


class Entry(NamedTuple):
    authors: list = []
    categories: list = []
    description: str = None
    id: str = None
    link: str = None
    published: str = None
    title: str = None
    updated: str = None



import logging
import os
from jinja2 import Environment, FileSystemLoader, select_autoescape
from pathlib import Path

from mkdocs.config import config_options
from mkdocs.plugins import BasePlugin
from mkdocs.structure.pages import Page

from . import __about__
from .feed import Entry
from .util import Util

FEED_FILENAME = "atom.xml"
TEMPLATE_FOLDER = Path(__file__).parent / "templates"
TEMPLATE_FILENAME = "atom.xml.jinja2"

logger = logging.getLogger("mkdocs.mkdocs_atom_plugin")


class AtomPlugin(BasePlugin):

    config_scheme = (
        ("abstract_chars_count", config_options.Type(int, default = 160)),
        ("datetime", config_options.Type(dict, default=None)),
        ("length", config_options.Type(int, default = 20)),
    )


    def __init__(self):
        self.util = Util()
        self.feed = {'generator_version': __about__.__version__}
        self.entries = []


    def on_config(self, config: config_options.Config) -> dict:
        self.feed['description'] = config.get("site_description", None)
        self.feed['entries'] = []
        self.feed['title'] = config.get("site_name", None)
        self.feed['updated'] = self.util.get_build_date()

        html_url = config.get("site_url", None)
        if html_url is not None:
            html_url = os.path.join(html_url, '') # add trailing slash
            self.feed['html_url'] = html_url
            self.feed["rss_url"] = html_url + FEED_FILENAME

        return config


    def on_page_content(self, html: str, page: Page, config: config_options.Config, files) -> str:
        if self.config.get('datetime') is not None:
            datetime_format = self.config.get('datetime').get('format', '%Y-%m-%d %H:%M')
            timezone_name = self.config.get('datetime').get('timezone', 'UTC')
        else:
            datetime_format = '%Y-%m-%d %H:%M'
            timezone_name = 'UTC'

        self.entries.append(
            Entry(
                authors = self.util.get_authors(page = page),
                categories = self.util.get_categories(page = page),
                description = self.util.get_abstract(page = page, chars_count = self.config.get("abstract_chars_count")),
                id = page.canonical_url,
                link = page.canonical_url,
                published = self.util.get_page_date(page = page, meta_date = 'published', datetime_format = datetime_format, timezone_name = timezone_name),
                title = page.title,
                updated = self.util.get_page_date(page = page, meta_date = 'updated', datetime_format = datetime_format, timezone_name = timezone_name, fallback = True),
            )
        )


    def on_post_build(self, config: config_options.Config) -> dict:
        length = self.config.get("length")
        feed_entries = sorted(self.entries, key = lambda entry: entry.updated, reverse = True)[:length]
        self.feed.get("entries").extend(feed_entries)

        env = Environment(
            autoescape = select_autoescape(["html", "xml"]),
            loader = FileSystemLoader(TEMPLATE_FOLDER),
        )

        template = env.get_template(TEMPLATE_FILENAME)

        out_path = Path(config.get("site_dir")) / FEED_FILENAME
        with out_path.open(mode = "w", encoding = "UTF8") as f:
            f.write(template.render(feed = self.feed))



import logging
from datetime import datetime, timedelta, timezone
from mimetypes import guess_type
from pathlib import Path
from typing import Iterable, Tuple
from urllib.parse import urlencode, urlparse, urlunparse

import markdown
from mkdocs.structure.pages import Page
from mkdocs.utils import get_build_timestamp


from .feed import Author, Category


logger = logging.getLogger("mkdocs.mkdocs_atom_plugin")


class Util:

    def get_authors(self, page: Page) -> list:
        ret = []

        if page.meta.get('author_name'):
            ret.append(Author(name = page.meta.get('author_name'), email = page.meta.get('author_email')))

        if page.meta.get('authors'):
            for author in page.meta.get('authors'):
                ret.append(Author(name = author['name'], email = author['email']))

        return ret


    def get_categories(self, page: Page) -> list:
        ret = []

        if page.meta.get('category_term'):
            ret.append(Category(term = page.meta.get('category_term'), label = page.meta.get('category_label')))

        if page.meta.get('categories'):
            for category in page.meta.get('categories'):
                ret.append(Category(term = category['term'], label = category['label']))

        return ret


    def to_string_with_tz(self, dt, timezone_name: str = 'UTC') -> str:
        if timezone_name.upper() == 'JST':
            tz = timezone(timedelta(hours=+9), 'JST')
        else:
            tz = timezone.utc

        dt = dt.replace(tzinfo=tz)
        return dt.isoformat()


    def get_build_date(self, timezone_name: str = 'UTC'):
        return self.to_string_with_tz(datetime.fromtimestamp(get_build_timestamp()), timezone_name = timezone_name)


    def get_page_date(self, page: Page, meta_date: str, datetime_format: str, timezone_name: str = 'UTC', fallback: bool = False) -> int:
        if page.meta.get(meta_date):
            ret = self.get_date_from_meta(meta_date_value = page.meta.get(meta_date), datetime_format = datetime_format)
            if ret is not None:
                return self.to_string_with_tz(ret, timezone_name = timezone_name)
            elif fallback:
                logging.warning(f"[atom-plugin] Front matter {meta_date} could not be retrieved for page: {page.file.abs_src_path}.")
                return self.get_build_date(timezone_name = timezone_name)
        elif fallback:
            logging.warning(f"[atom-plugin] Front matter {meta_date} is not defined on page: {page.file.abs_src_path}.")
            return self.get_build_date(timezone_name = timezone_name)

        return None


    def get_date_from_meta(self, meta_date_value: str, datetime_format: str) -> float:
        try:
            if isinstance(meta_date_value, str):
                return datetime.strptime(meta_date_value, datetime_format)
            else:
                logging.warning("[atom-plugin] Incompatible date type.")
        except ValueError as err:
            logging.warning(f"[atom-plugin] Incompatible date found. Trace: {err}")
        except Exception as err:
            logging.warning(f"[atom-plugin] Unable to retrieve date. Trace: {err}")

        return None


    def get_abstract(self, page: Page, chars_count: int = 160) -> str:
        if chars_count < 0:
            chars_count = None

        if page.markdown:
            if chars_count is None or len(page.markdown) < chars_count:
                return markdown.markdown(page.markdown[:chars_count], output_format = "html5")
            else:
                return markdown.markdown(f"{page.markdown[: chars_count - 3]}...", output_format = "html5")
        else:
            return None

